;******************************************************************************
;* Beispiel für cooperative Multitasking Funktionen des avrmultos
;* @author: (c) Markus STRUTZENBERGER, backwork
;* @date: 2005-09-01
;* @target:Atmel AVR ATtiny26(L)
;*
;* Nomenklatur:
;* Großgeschriebene Labels markieren Funktionen die mit CALL aufgerufen werden müssen.
;* Kleingeschriebene Labels markieren Sprungmarken für RJMP
;*
;* GPL:
;* This program is free software; you can redistribute it and/or modify
;* it under the terms of the GNU General Public License as published by
;* the Free Software Foundation; either version 2 of the License, or
;* (at your option) any later version.
;*
;* This program is distributed in the hope that it will be useful,
;* but WITHOUT ANY WARRANTY; without even the implied warranty of
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;* GNU General Public License for more details.
;*
;* You should have received a copy of the GNU General Public License
;* along with this program; if not, write to the Free Software
;* Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
;*
;******************************************************************************

.DEF rTmp=R16
.DEF rTask1=R23
.DEF rTask2=R24
.DEF rTask3=R25

; Start des Programms
.CSEG
;******************************************************************************
;* Interrupt Tabelle
;******************************************************************************
.ORG 0x0000         ; Addr for ATtiny26
  RJMP startup      ; 00 reset
  reti              ; 01 int0
  reti              ; 02 I/O pins
  reti              ; 03 T1 CMPA
  reti              ; 04 T1 CMPB
  reti              ; 05 Timer1 Ovf
  RJMP MosTimer0ISR ; 06 Timer0 Ovf
  reti              ; 07 USI start
  reti              ; 08 USI Ovf
  reti              ; 09 EEPROM rdy
  reti              ; 0A ANA_COMP
  reti              ; 0B ADC complete

.INCLUDE "avrmultos.inc"

;******************************************************************************
;* Reset Funktion
;******************************************************************************
startup:
  ; Stackpointer initialisieren
  ldi rtmp, RAMEND
  out SP, rTmp

  rcall InitIO
  rcall MosInit
  
  ldi rMosParam16L, low(TestFn1)
  ldi rMosParam16H, high(TestFn1)
  rcall MosAddTask
  ldi rMosParam16L, low(TestFn2)
  ldi rMosParam16H, high(TestFn2)
  rcall MosAddTask
  ldi rMosParam16L, low(TestFn3)
  ldi rMosParam16H, high(TestFn3)
  rcall MosAddTask

  rjmp mosStartScheduler ; auf nimmerwiedersehen!

;******************************************************************************
;* init
;******************************************************************************
InitIO:
  ldi rTmp, 0b00111111
  out DDRA, rTmp
  ret
;******************************************************************************


;******************************************************************************
;* Testfunktion1
;******************************************************************************
TestFn1:
  dec rTask1
  mov rTmp, rTask1
  andi rTmp, 0x01
  breq fn1_on
  sbi PORTA,1
  rjmp fn1_off
fn1_on:
  cbi PORTA,1
fn1_off:
  rcall MosNextTask
  rjmp TestFn1
;******************************************************************************

;******************************************************************************
;* Testfunktion2
;******************************************************************************
TestFn2:
  dec rTask2
  mov rTmp, rTask2
  andi rTmp, 0x01
  breq fn2_on
  sbi PORTA,2
  rjmp fn2_off
fn2_on:
  cbi PORTA,2
fn2_off:
  ldi R26, 0x14
  ldi R27, 0x00
;X  rcall MosSleep
  rjmp TestFn2
;******************************************************************************

;******************************************************************************
;* Testfunktion3
;******************************************************************************
TestFn3:
  dec rTask3
  mov rTmp, rTask3
  andi rTmp, 0x01
  breq fn3_on
  cbi PORTA,3
  rjmp fn3_off
fn3_on:
  sbi PORTA,3
fn3_off:
  ldi R26, 0x60
  ldi R27, 0xea
  rcall MosSleep
  rjmp TestFn3
;******************************************************************************
